/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.util.swing;

import javax.swing.ImageIcon;
import javax.swing.JPanel;
import org.openide.util.Lookup;

/**
 * Decorate another panel with standard header, footer, etc.
 * Extracted superclass from ASBodyPanel to allow for multiple implementations.
 * Register a BodyPanel.Factory implementation as a ServiceProvider to use a
 * custom BodyPanels in your application instead of the default ASBodyPanel.
 * 
 * @author johan
 */
public abstract class BodyPanel extends JPanel {
    private static final Factory FACTORY = Lookup.getDefault().lookup(BodyPanel.Factory.class);
    
    public abstract void setTitle(String title);
    
    public abstract void setIcon(ImageIcon icon);
    
    public static BodyPanel create(JPanel toDecorate) {
        return FACTORY != null ? FACTORY.create(toDecorate) : new ASBodyPanel(toDecorate);
    }
    
    public static interface Factory {
        public BodyPanel create(JPanel toDecorate);
    }
}
