/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.util.swing;

import java.awt.Component;
import java.util.HashSet;
import java.util.Set;
import javax.swing.ComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JList;
import javax.swing.UIManager;
import javax.swing.plaf.basic.BasicComboBoxRenderer;

/**
 *
 * @author merijn
 */
public class DisabledItemsComboBox<T> extends JComboBox<T> {

    private Set disabled_items = new HashSet();

    public DisabledItemsComboBox() {
        super();
        super.setRenderer(new DisabledItemsRenderer());
    }

    public DisabledItemsComboBox(ComboBoxModel<T> boxModel) {
        super(boxModel);
        super.setRenderer(new DisabledItemsRenderer());
    }

    public void addItem(T anObject, boolean disabled) {
        super.addItem(anObject);
        if (disabled) {
            disabled_items.add(getItemCount() - 1);
        }
    }

    public void setDisabled(int index) {
        disabled_items.add(index);
    }

    public void setDisabled(T anObject) {
        for (int i = 0; i < getItemCount(); i++) {
            if (anObject.equals(getItemAt(i))) {
                disabled_items.add(i);
            }
        }
    }

    @Override
    public void removeAllItems() {
        super.removeAllItems();
        disabled_items = new HashSet();
    }

    @Override
    public void removeItemAt(final int anIndex) {
        super.removeItemAt(anIndex);
        disabled_items.remove(anIndex);
    }

    @Override
    public void removeItem(final Object anObject) {
        for (int i = 0; i <= getItemCount(); i++) {
            if (getItemAt(i) == anObject) {
                disabled_items.remove(i);
            }
        }
        super.removeItem(anObject);
    }

    @Override
    public void setSelectedIndex(int index) {
        if (!disabled_items.contains(index)) {
            super.setSelectedIndex(index);
        }
    }

    private class DisabledItemsRenderer extends BasicComboBoxRenderer {

        @Override
        public Component getListCellRendererComponent(JList list,
                Object value,
                int index,
                boolean isSelected,
                boolean cellHasFocus) {

            if (isSelected) {
                setBackground(list.getSelectionBackground());
                setForeground(list.getSelectionForeground());
            } else {
                setBackground(list.getBackground());
                setForeground(list.getForeground());
            }
            if (disabled_items.contains(index)) {
                setBackground(list.getBackground());
                setForeground(UIManager.getColor("Label.disabledForeground"));
            }
            setFont(list.getFont());
            setText((value == null) ? "" : value.toString());
            return this;
        }
    }
}
