/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.util.swing;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.LayoutManager;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.KeyEvent;
import java.io.Serializable;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 *
 * @author Timon Veenstra
 */
public class InlineEditLabel extends JPanel implements Serializable{

    public enum State {

        LABEL, EDIT
    }
    private final JLabel label;
    private final JTextField textField = new JTextField();
    private State state = State.LABEL;

    protected JLabel getLabel() {
        return label;
    }

    public InlineEditLabel() {
        super(null);
        setPreferredSize(new Dimension(220, 26));
        //
        // label
        //
        label = new JLabel();
        label.setPreferredSize(new Dimension(220, 20));
        label.addMouseListener(new java.awt.event.MouseAdapter() {

            public void mouseClicked(java.awt.event.MouseEvent evt) {
                nameLabelMouseClicked(evt);
            }
        });
        label.setVisible(true);
        add(label);

        //
        // textfield
        //        
        textField.setPreferredSize(getPreferredSize());
        textField.addFocusListener(new java.awt.event.FocusAdapter() {

            public void focusLost(java.awt.event.FocusEvent evt) {
                nameTextFocusLost(evt);
            }
        });
        textField.addKeyListener(new java.awt.event.KeyAdapter() {

            public void keyPressed(java.awt.event.KeyEvent evt) {
                nameTextKeyPressed(evt);
            }
        });

        add(textField);
        textField.setVisible(false);
        
        addComponentListener(new ComponentListener() {

            @Override
            public void componentResized(ComponentEvent e) {
                Dimension newSize = getSize();
                label.setSize(new Dimension(newSize.width, newSize.height-4));
                textField.setSize(newSize);
            }

            @Override
            public void componentMoved(ComponentEvent e) {
            }

            @Override
            public void componentShown(ComponentEvent e) {
            }

            @Override
            public void componentHidden(ComponentEvent e) {
            }
        });
    }

    private void nameLabelMouseClicked(java.awt.event.MouseEvent evt) {
        setState(State.EDIT);
    }

    public void setValue(String newValue) {
        label.setText(newValue);
        textField.setText(newValue);
    }

    public String getValue() {
        return label.getText();
    }

    private void processNameChange() {
        setState(State.LABEL);
    }

    private void nameTextFocusLost(java.awt.event.FocusEvent evt) {
        processNameChange();
    }

    private void nameTextKeyPressed(java.awt.event.KeyEvent evt) {
        if (evt.getKeyCode() == KeyEvent.VK_ENTER) {
            processNameChange();
        }
    }

    public State getState() {
        return state;
    }

    public void setState(State state) {
        this.state = state;
        switch (state) {
            case LABEL:
                label.setText(textField.getText());
                label.setVisible(true);
                textField.setVisible(false);
                setValue(textField.getText());
                break;
            case EDIT:
                label.setVisible(false);
                textField.setVisible(true);
                textField.setSelectionStart(0);
                textField.setSelectionEnd(textField.getText().length());
                textField.requestFocus();
                break;
            default:
        }
    }
}
